"use strict";
/**
 * Copyright (c) 2022 - present Tiny CLI Authors.
 * Copyright (c) 2022 - present Huawei Cloud Computing Technologies Co., Ltd.
 *
 * Use of this source code is governed by an MIT-style license.
 *
 * THE OPEN SOURCE SOFTWARE IN THIS PRODUCT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
 * A PARTICULAR PURPOSE. SEE THE APPLICABLE LICENSES FOR MORE DETAILS.
 *
 */
/**
 * 查看 tiny 及 套件的 帮助信息
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_devkit_1 = require("@opentiny/cli-devkit");
const chalk_1 = __importDefault(require("chalk"));
const index_1 = __importDefault(require("./locale/index"));
/**
 * 显示tiny帮助
 */
function outHelpInfo(needToolkit) {
    const intl = new cli_devkit_1.Intl(index_1.default);
    const tool = cli_devkit_1.cliConfig.getBinName();
    const help = intl.get('help', { tool });
    // 打印帮助信息
    console.log(chalk_1.default.cyan(help));
    console.log(chalk_1.default.yellow(intl.get('helpTips')));
    needToolkit && console.log(chalk_1.default.yellow(intl.get('helpToolkit')));
    console.log(chalk_1.default.yellow(intl.get('helpPlugin', { tool })));
}
function isGenerator(obj) {
    return typeof obj.next === 'function' && typeof obj.throw === 'function';
}
/**
 * 判断当前对象是否为 generator 函数
 * @param obj
 * @returns {boolean}
 */
function isGeneratorFunction(obj) {
    const constructor = obj.constructor;
    if (!constructor) {
        return false;
    }
    if (constructor.name === 'GeneratorFunction' || constructor.displayName === 'GeneratorFunction') {
        return true;
    }
    return isGenerator(constructor.prototype);
}
async function default_1() {
    const toolkit = cli_devkit_1.config.getToolkitName();
    const intl = new cli_devkit_1.Intl(index_1.default);
    const tool = cli_devkit_1.cliConfig.getBinName();
    // 套件存在,则优先输出套件帮助信息
    if (toolkit) {
        const mod = await cli_devkit_1.modules.get(toolkit);
        const help = cli_devkit_1.modules.getEsModule(mod.help);
        if (help) {
            if (isGeneratorFunction(help)) {
                await help();
            }
            else {
                help();
            }
            console.log(chalk_1.default.cyan(intl.get('helpList', { tool })));
        }
        outHelpInfo();
    }
    else {
        outHelpInfo(true);
    }
}
exports.default = default_1;
