"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2022 - present Tiny CLI Authors.
 * Copyright (c) 2022 - present Huawei Cloud Computing Technologies Co., Ltd.
 *
 * Use of this source code is governed by an MIT-style license.
 *
 * THE OPEN SOURCE SOFTWARE IN THIS PRODUCT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
 * A PARTICULAR PURPOSE. SEE THE APPLICABLE LICENSES FOR MORE DETAILS.
 *
 */
const cli_devkit_1 = require("@opentiny/cli-devkit");
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const inquirer_1 = __importDefault(require("inquirer"));
const yargs_1 = __importDefault(require("yargs"));
const utils_1 = require("./utils");
const index_1 = __importDefault(require("./locale/index"));
const clientOptions = Object.assign({}, yargs_1.default.help(false).argv);
const log = cli_devkit_1.logs('core-commands');
const cwd = process.cwd();
async function runInit(name) {
    const module = await cli_devkit_1.modules.getReallyName(name);
    const intl = new cli_devkit_1.Intl(index_1.default);
    if (module.exist) {
        const moduleInfo = await cli_devkit_1.modules.get(module.reallyName);
        // es module default export xx.default
        // moduleInfo = modules.getEsModule(moduleInfo);
        // 存在init方法的话，则调用
        if (moduleInfo.init) {
            const init = cli_devkit_1.modules.getEsModule(moduleInfo.init);
            // 去掉 clientOptions 里面多余的字段
            delete clientOptions._;
            delete clientOptions.$0;
            // 执行插件的init方法
            await cli_devkit_1.task.runFunction({
                method: init,
                args: [clientOptions],
            });
        }
        else {
            log.error(intl.get('notMethod'));
        }
    }
    else {
        const msg = intl.get('toolkitNotFound', { toolkit: module.reallyName });
        log.error(msg);
    }
}
async function getName() {
    const choices = [];
    const onlineList = await cli_devkit_1.modules.onlineList({ type: 'toolkit' });
    const localList = cli_devkit_1.modules.localList({ type: 'toolkit' });
    const intl = new cli_devkit_1.Intl(index_1.default);
    const onlineMap = {};
    const addChoice = (item) => {
        const n = item.name.split('-toolkit-')[1];
        const padding = utils_1.getPadding(n, 15);
        const tmpString = ['  ', chalk_1.default.green(n), chalk_1.default.gray(padding), item.chName ? item.chName : '暂无描述'].join('');
        choices.push({
            name: tmpString,
            value: item.name,
        });
    };
    onlineList.forEach((item) => {
        addChoice(item);
        onlineMap[item.name] = true;
    });
    localList.forEach((item) => {
        if (!onlineMap[item.name]) {
            addChoice(item);
        }
    });
    // 本地与远程无可用套件时
    if (!onlineList.length && !localList.length) {
        log.error(intl.get('noData'));
        return process.exit(1);
    }
    const answers = await inquirer_1.default.prompt([
        {
            type: 'list',
            name: 'name',
            message: intl.get('toolkitInit'),
            choices,
        },
    ]);
    return answers.name;
}
async function default_1(args) {
    let name = args.pop() || '';
    const intl = new cli_devkit_1.Intl(index_1.default);
    if (!name) {
        // 未传入套件名,提示并列出可用套件名
        name = await getName();
    }
    name = cli_devkit_1.modules.utils.toolkitFullName(name);
    // 先判断tiny.config.js 是否存在
    // 存在的话,提示已初始化过了
    // 不存在的话再判断文件夹是否为空
    // 不为空的话则提示覆盖
    if (cli_devkit_1.config.exist(cwd)) {
        log.warn(intl.get('toolkitReportInit'));
        log.warn(intl.get('toolkitInitTips', { file: cli_devkit_1.config.getConfigName() }));
        return;
    }
    // 排除那些以点开头的文件
    const files = fs_extra_1.default.readdirSync(cwd).filter((file) => file.indexOf('.') !== 0);
    if (files.length > 0) {
        log.warn(intl.get('fileExist'));
        const questions = [
            {
                type: 'input',
                name: 'check',
                message: intl.get('confirmInit'),
            },
        ];
        const answers = await inquirer_1.default.prompt(questions);
        if (answers.check === 'y' || answers.check === 'Y') {
            await runInit(name);
        }
    }
    else {
        await runInit(name);
    }
}
exports.default = default_1;
