"use strict";
/**
 * Copyright (c) 2022 - present Tiny CLI Authors.
 * Copyright (c) 2022 - present Huawei Cloud Computing Technologies Co., Ltd.
 *
 * Use of this source code is governed by an MIT-style license.
 *
 * THE OPEN SOURCE SOFTWARE IN THIS PRODUCT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
 * A PARTICULAR PURPOSE. SEE THE APPLICABLE LICENSES FOR MORE DETAILS.
 *
 */
/**
 * @desc: 列出所有的套件,插件
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const cli_devkit_1 = require("@opentiny/cli-devkit");
const chalk_1 = __importDefault(require("chalk"));
const index_1 = __importDefault(require("./locale/index"));
const utils_1 = require("./utils");
const log = cli_devkit_1.logs('core-commands');
function printListByType(type, moduleList) {
    let tmpString;
    moduleList
        .filter((item) => !!item.name.match(type))
        .forEach((item) => {
        const padding = utils_1.getPadding(item.name, 35);
        tmpString = ['  ', chalk_1.default.green(item.name), chalk_1.default.gray(padding), item.chName ? item.chName : '暂无描述'].join('');
        console.log(tmpString);
    });
}
async function default_1(cliArgs, _options) {
    const type = cliArgs.pop();
    const intl = new cli_devkit_1.Intl(index_1.default);
    const fixType = type === 'plugin' || type === 'toolkit' ? type : null;
    const textMap = {
        plugin: intl.get('plugin'),
        toolkit: intl.get('toolkit'),
        all: intl.get('toolkitAndPlugin'),
    };
    const text = textMap[fixType || 'all'];
    const star = fixType ? '**' : '';
    const param = { fixType };
    log.debug('module params = %o', param);
    const local = cli_devkit_1.modules.localList(param);
    const online = await cli_devkit_1.modules.onlineList(param);
    const newList = [];
    // merge list
    const onlineKeys = online.map((item) => {
        newList.push(item);
        return item.name;
    });
    local.forEach((item) => {
        if (onlineKeys.indexOf(item.name) === -1) {
            newList.push(item);
        }
    });
    console.log(chalk_1.default.italic.magenta(`\r\n${star}************** ${text} ${intl.get('list')} ******************${star}\r\n`));
    if (!type) {
        console.log(chalk_1.default.magenta(intl.get('toolkitList')));
        printListByType('toolkit', newList);
        console.log(chalk_1.default.magenta(intl.get('pluginList')));
        printListByType('plugin', newList);
    }
    else {
        printListByType(type, newList);
    }
    console.log(chalk_1.default.italic.magenta('\r\n***************************************************\r\n'));
}
exports.default = default_1;
