"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Copyright (c) 2022 - present Tiny CLI Authors.
 * Copyright (c) 2022 - present Huawei Cloud Computing Technologies Co., Ltd.
 *
 * Use of this source code is governed by an MIT-style license.
 *
 * THE OPEN SOURCE SOFTWARE IN THIS PRODUCT IS DISTRIBUTED IN THE HOPE THAT IT WILL BE USEFUL,
 * BUT WITHOUT ANY WARRANTY, WITHOUT EVEN THE IMPLIED WARRANTY OF MERCHANTABILITY OR FITNESS FOR
 * A PARTICULAR PURPOSE. SEE THE APPLICABLE LICENSES FOR MORE DETAILS.
 *
 */
/**
 * @desc: 所有的套件/插件公共流程
 * 自定义prefix 本地套件/插件 -> tiny本地套件/插件 -> 自定义线上套件/插件 -> tiny线上套件/插件
 */
const cli_devkit_1 = require("@opentiny/cli-devkit");
const chalk_1 = __importDefault(require("chalk"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const path = __importStar(require("path"));
const yargs_1 = __importDefault(require("yargs"));
const index_1 = __importDefault(require("./locale/index"));
const log = cli_devkit_1.logs('core-commands');
const clientOptions = Object.assign({}, yargs_1.default.parserConfiguration({ 'camel-case-expansion': false }).help(false).argv);
/**
 * 当遇到 start , build 命令时,判断用户是否在正确的目录
 * @param command
 * @returns {boolean}
 */
function isErrorDirectory(command) {
    const intl = new cli_devkit_1.Intl(index_1.default);
    // 如果当前目录下不存在tiny.config.js 则提示
    if (['start', 'build'].indexOf(command) !== -1 && !cli_devkit_1.config.exist()) {
        log.debug('error directory');
        log.error(intl.get('configFileNotFound', { file: cli_devkit_1.config.getConfigName() }));
        return false;
    }
    return true;
}
class Main {
    /**
     * 主流程(非内置核心命令主流程)
     * @param command
     * @param cliArgs
     */
    async start(command, cliArgs) {
        const tasks = cli_devkit_1.config.get('tasks') || {};
        const hasBeforeTask = cli_devkit_1.task.has(tasks[command], 'before');
        const hasAfterTask = cli_devkit_1.task.has(tasks[command], 'after');
        const intl = new cli_devkit_1.Intl(index_1.default);
        log.debug('tasks = %o , command = %s, cliArgs = %o', tasks, command, cliArgs);
        log.debug(`before task ${hasBeforeTask}`);
        // 去掉 clientOptions 里面多余的字段
        delete clientOptions._;
        delete clientOptions.$0;
        // 错误提示提前判断
        if (!isErrorDirectory(command)) {
            return;
        }
        // 如果第一个参数为 plugin, 强制执行某个插件, 并且忽略所有的前置,后置任务
        if (command === 'plugin') {
            if (cliArgs.length < 1) {
                log.error(intl.get('runPlugin'));
                return;
            }
            command = cliArgs.splice(0, 1)[0];
            log.debug('new tasks = %o , command = %s, cliArgs = %o', tasks, command, cliArgs);
            await this.runPlugin(command, cliArgs);
            return;
        }
        // ------------- 展示版本号, 并中止后面的任务 ---------------
        if (cliArgs && cliArgs.length === 0 && (clientOptions.v || clientOptions.version)) {
            await this.showVersion(command);
            return;
        }
        // ------------- 执行前置任务 ---------------
        await this.runPreTask(command, cliArgs);
        // -------------- 执行套件任务 ---------------
        let toolkitName = cli_devkit_1.config.exist() ? cli_devkit_1.config.get('toolkit') : '';
        let toolkit = {};
        if (toolkitName) {
            const moduleInfo = await cli_devkit_1.modules.getReallyName(toolkitName);
            // 判断套件是否存在
            if (moduleInfo.exist) {
                toolkitName = moduleInfo.reallyName;
                toolkit = await cli_devkit_1.modules.get(toolkitName);
            }
        }
        const toolkitCommand = cli_devkit_1.modules.getEsModule(toolkit[command]);
        // 如果判断到有套件且有对应命令的方法,那么直接执行并返回, 否则向下执行插件逻辑
        if (typeof toolkitCommand === 'function') {
            log.debug(`找到套件 ${toolkitName} 对应的 ${command} 方法`);
            // 套件发送log
            // log.debug(`套件 ${toolkitName} LOG开始发送...`);
            cli_devkit_1.cliConfig.setModuleEntry(toolkitName);
            const afterToolCommand = async () => {
                // -------------- 执行后置任务 ---------------
                await this.runPostTask(command, cliArgs);
            };
            // 传入 callback ,兼容未使用 generator 版本套件和插件
            const optionsArg = {
                clientArgs: cliArgs,
                clientOptions,
            };
            await cli_devkit_1.task.runFunction({
                method: toolkitCommand,
                args: toolkitCommand.length > 1 ? [optionsArg, afterToolCommand] : [optionsArg],
                // task 模块调用
                next: afterToolCommand,
            });
            return;
        }
        else if (hasAfterTask) {
            log.debug('未找到对应的套件及方法');
            // 只有后置命令, 却没有套件模块的给个提示
            const msg = intl.get('notRunTips', { command });
            log.error(msg);
            return;
        }
        // start build 错误提示
        if (['start', 'build'].indexOf(command) !== -1) {
            if (toolkitName) {
                const tool = cli_devkit_1.cliConfig.getBinName();
                log.error(intl.get('startNotRunTips', { command, tool }));
            }
            else {
                // 存在tiny.config.js文件且文件中有对应的 start、build、publish时则不需要提示
                if (!(hasBeforeTask || hasAfterTask)) {
                    log.error(intl.get('configNotRunTips', {
                        file: cli_devkit_1.config.getConfigName(),
                        command,
                    }));
                }
            }
            return;
        }
        // -------------- 执行插件任务 ---------------
        // 在已经执行了任务流的情况下,直接不执行插件逻辑
        if (!hasBeforeTask && !hasAfterTask) {
            log.debug('尝试执行插件方法');
            await this.runPlugin(command, cliArgs);
        }
    }
    /**
     * 执行前置任务
     * @param command
     * @param cliArgs
     */
    async runPreTask(command, cliArgs) {
        const tasks = cli_devkit_1.config.get('tasks') || {};
        const hasBeforeTask = cli_devkit_1.task.has(tasks[command], 'before');
        if (hasBeforeTask) {
            const optionsArg = {
                clientArgs: cliArgs,
                clientOptions,
            };
            await cli_devkit_1.task.run({
                tasks: tasks[command],
                args: [
                    Object.assign({}, optionsArg),
                    optionsArg,
                ],
                when: 'before',
                command,
            });
        }
    }
    /**
     * 执行后置任务
     * @param command
     * @param cliArgs
     */
    async runPostTask(command, cliArgs) {
        const tasks = cli_devkit_1.config.get('tasks') || {};
        const hasAfterTask = cli_devkit_1.task.has(tasks[command], 'after');
        if (hasAfterTask) {
            try {
                const optionsArg = {
                    clientArgs: cliArgs,
                    clientOptions,
                };
                await cli_devkit_1.task.run({
                    tasks: tasks[command],
                    args: [optionsArg],
                    when: 'after',
                    command,
                });
            }
            catch (err) {
                cli_devkit_1.error(err);
            }
        }
    }
    /**
     * 运行插件命令
     * 运行逻辑：自定义本地插件 -> tiny本地插件 -> 自定义线上插件 -> tiny线上插件
     * 先走本地已安装，速度快一些
     * @param name
     * @param cliArgs
     */
    async runPlugin(name, cliArgs) {
        const fullName = cli_devkit_1.modules.utils.pluginFullName(name);
        const module = await cli_devkit_1.modules.getReallyName(fullName);
        const intl = new cli_devkit_1.Intl(index_1.default);
        if (module.exist) {
            // 设置模块入口
            cli_devkit_1.cliConfig.setModuleEntry(module.reallyName);
            const plugin = await cli_devkit_1.modules.get(module.reallyName);
            let method;
            let pluginCmd = '';
            log.debug(' 插件信息 %o', plugin);
            if (typeof plugin === 'function') {
                method = plugin;
            }
            else if (typeof plugin === 'object') {
                if (cliArgs.length) {
                    pluginCmd = cliArgs.shift();
                    method = cli_devkit_1.modules.getEsModule(plugin[pluginCmd]);
                }
                else if (typeof plugin.default === 'function') {
                    method = plugin.default;
                }
            }
            if (!method) {
                const msg = intl.get('pluginCommandNotFound', {
                    module: module.reallyName,
                    pluginCmd,
                });
                log.error(msg);
                return;
            }
            const optionsArg = { clientArgs: cliArgs, clientOptions };
            await cli_devkit_1.task.runFunction({
                method,
                args: [optionsArg],
            });
        }
        else {
            const msg = intl.get('pluginNotFound', { plugin: module.reallyName });
            log.error(msg);
        }
    }
    /**
     * 展示本地版本号，显示查找逻辑：自定义prefix本地模块 -> tiny本地模块
     */
    async showVersion(name) {
        let existsOne = false;
        const intl = new cli_devkit_1.Intl(index_1.default);
        const logOne = async function (n) {
            n = cli_devkit_1.modules.utils.fullName(n);
            const prefix = cli_devkit_1.modules.utils.prefix();
            const localExist = cli_devkit_1.modules.localExist(n);
            let mod = '';
            if (localExist) {
                mod = fs_extra_1.default.readJsonSync(path.resolve(cli_devkit_1.home.getModulesPath(), n, 'package.json'), {
                    throws: false,
                });
            }
            else if (prefix !== cli_devkit_1.cliConfig.DEFAULT_BIN) {
                n = n.replace(prefix, cli_devkit_1.cliConfig.DEFAULT_BIN);
                mod = fs_extra_1.default.readJsonSync(path.resolve(cli_devkit_1.home.getModulesPath(), n, 'package.json'), {
                    throws: false,
                });
            }
            if (mod && mod.version) {
                existsOne = true;
                console.log(chalk_1.default.magenta(intl.get('moduleVersion', { module: n, version: mod.version })));
            }
        };
        if (name.indexOf('toolkit-') > -1 || name.indexOf('plugin-') > -1) {
            await logOne(name);
            return;
        }
        await logOne(`toolkit-${name}`);
        await logOne(`plugin-${name}`);
        if (!existsOne) {
            const msg = intl.get('localNotFound', { name });
            log.error(msg);
        }
    }
}
const main = new Main();
exports.default = (command, cliArgs) => {
    main.start(command, cliArgs);
};
